<?php
namespace Api\Helper;

class SummaryFactory {

    public function createSummary($results) {
    }

    public function parseBadges($results) {
    }
    
    public function parseDistances($results) {
    }
    
}

