<?php

class test {

    private $privProp;
    public $pubProp;
    protected $protProp;

    public function thisIsPublic() { }
    private function thisIsPrivate() { }
    protected function thisIsProtected() { }

}
