<?php
namespace {

    /**
     * Class MyClass
     */
    class OtherClass {

        /*
         * bytes '0xA0 0x7B 0x40 0x69'
         */
        const value = '<?xml version="1.0" encoding="utf-8" ?><r>�{@i</r>';

    }

}
