<?php

namespace TheSeer\phpDox\Generator\Enricher {

    use TheSeer\fDOM\fDOMElement;
    use TheSeer\phpDox\GeneratorConfig;

    class PHPCsConfig extends CheckStyleConfig{

        protected $defaultFileName = 'phpcs.xml';

    }

}
