<?php

class ValidASCII {

    /**
     * Check if the current product or oder item
     * or quote item have a certificate option enabled
     *
     * @param $productId int
     * @param Mage_Sales_Model_Order_Item |a Mage_Sales_Model_Quote_Item $item
     * @return bool
     */
    public function b($productId, $item) {
    }

}
